 -- FailBot w/ Support for : LDB-1.1, Sink-2.0, and configuration via AceOptions

-- Credits to FailBot by Veev for the original code used to make libfail.
local ADDON_NAME = "FailBoat"

local addon = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "LibSink-2.0", "AceConsole-3.0")

local L	= LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)

local media = LibStub("LibSharedMedia-3.0") 
local fail = LibStub("LibFail-1.0")

local defaults = {
	profile = {
		sinkoptions = { --[["sink20OutputSink"] = "Popup"]] },
		sound = nil,
	},
    realm = {
        fail_counts = { ["*"] = { ["*"] = 0 } }
    }
}


--[===[@debug@ 
local function dbg(...)
    if Prat then
        Prat.CustomPrint(addon, nil, nil, nil, nil, nil, true, ...)
    else
        print(...)
    end
end
--@end-debug@]===]

function addon:SetValue(info, b)
	self.db.profile[info[#info]] = b
end
function addon:GetValue(info)
	return self.db.profile[info[#info]]
end

local curr_fail = 0

local options = {
    name = L["addon_name"],
    desc = L["addon_desc"],
    type = "group",
    get = "GetValue",
    set = "SetValue", 
    handler = addon,
    args = {
		output = addon:GetSinkAce3OptionsDataTable(),
	
		sound = {
            name = L["sound_name"],
            desc = L["sound_desc"],
 			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
		},
    },
}

options.args.output.inline = true

local DO = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(ADDON_NAME, {
		type = "data source",
		text = L["addon_name"],
        icon = [[Interface\ICONS\Spell_Arcane_MindMastery]],
})


function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("FailBoatDB", defaults, "Default")

	self:SetSinkStorage(self.db.profile.sinkoptions)

	local acreg = LibStub("AceConfigRegistry-3.0")
	
	acreg:RegisterOptionsTable(ADDON_NAME, options)
	
	local acdia = LibStub("AceConfigDialog-3.0")
	
	acdia:AddToBlizOptions(ADDON_NAME, ADDON_NAME)
	
	addon:RegisterChatCommand(ADDON_NAME:lower(), function() acdia:Open(ADDON_NAME) end)
end


local fail_events = fail:GetSupportedEvents()

local fail_counts

function addon:OnEnable()
    fail_counts = self.db.realm.fail_counts or {}
    self.db.realm.fail_counts = fail_counts

	for _, v in pairs(fail_events) do
		fail.RegisterCallback(self, v, "Fail")
        fail_counts[v] = fail_counts[v] or {}
	end

	media.RegisterCallback(self, "LibSharedMedia_Registered", "SharedMedia_Registered")
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "SharedMedia_Registered")

    fail.RegisterCallback(self, "Fail_Active")
    fail.RegisterCallback(self, "Fail_Inactive")

    if fail.active then 
        self:Fail_Active()
    else
        self:Fail_Inactive()
    end        
end


function addon:Fail_Clear()
    self.db.realm.fail_counts = {}
    fail_counts = self.db.realm.fail_counts
	for _, v in pairs(fail_events) do
        fail_counts[v] = {}
	end
    print("Fail Counts Reset")
end

function addon:Fail_Active()
   DO.text =  L["addon_name"]..L["addon_active"]
end

function addon:Fail_Inactive()
   DO.text =  L["addon_name"]..L["addon_inactive"]
end

function addon:OnDisable()
	fail.UnregisterAllEvents(self)
	media.UnregisterAllCallbacks(self)
end

local SOUND = media.MediaType.SOUND
local count

function addon:Fail(event, name, ...)
    fail_counts = self.db.realm.fail_counts
    count = (fail_counts[event][name] or 0) + 1
    fail_counts[event][name] = count


--[===[@debug@ 
    dbg(event, name, ...)
--@end-debug@]===]
	local txt = L[event]:format(name, ...)

    -- add a count to the end
    txt = txt..(L["fail_count"]):format(count)

--[===[@debug@ 
    dbg(txt)
--@end-debug@]===]

	if self.db.profile.sound then

		PlaySoundFile(media:Fetch(SOUND, self.db.profile.sound))
	end

	self:Pour(txt)
end


function addon:SharedMedia_Registered(event, mediatype, name)
    if mediatype == SOUND then
        -- Update list of sounds?
    end
end

local function toggleOptionsWindow()
    local acd = LibStub("AceConfigDialog-3.0")
    if acd.OpenFrames[ADDON_NAME] then
        acd:Close(ADDON_NAME)
    else
        acd:Open(ADDON_NAME)
    end
end

function DO.OnClick(frame, button)
	if button == "RightButton" then
        toggleOptionsWindow()
	else
		if IsShiftKeyDown() then
--			print("shift-click")
--[===[@debug@ 
            fail:Test()
            DO.OnEnter(frame)
--@end-debug@]===]
		elseif IsAltKeyDown() then
            addon:Fail_Clear()
            DO.OnEnter(frame)
		else
            curr_fail = curr_fail + 1
            curr_fail = math.fmod(curr_fail, #fail_events + 1)
            DO.OnEnter(frame)
		end
	end
end

local LTT = LibStub:GetLibrary( "LibQTip-1.0" )
local tooltip

local function Color(c, t) return "|cff"..c..t.."|r" end
local function HGreen(t) return Color("80ff80", t) end
local function HYellow(t) return Color("ffffa0", t) end
local function Green(t) return Color("80ff80", t) end
local function Blue(t) return Color("8080ff", t) end

local per_person = {}
local sort_table = {}

local function sortbyFails(t)
    wipe(sort_table)
    for name, count in pairs(t) do
        sort_table[#sort_table+1] = name
    end

    table.sort(sort_table, function(a,b) if t[a] == t[b] then return a < b else return t[a] > t[b] end end )
    
    return sort_table
end

function DO.OnEnter(self)
    tooltip = LTT:Acquire(ADDON_NAME, 2, "LEFT", "RIGHT")

    tooltip:Clear()
    fail_counts = addon.db.realm.fail_counts

    wipe(per_person)

    local total, grandtotal, expand, empty = 0,0,0
    for event,counts in pairs(fail_counts) do
        total, expand = 0, expand+1
        for name,count in pairs(counts) do total = total + count end
        grandtotal = grandtotal + total
    	tooltip:AddHeader(HGreen(L["label_"..event]), HGreen(tostring(total)))
        empty = true
        local t
        if expand == curr_fail then
            t = sortbyFails(counts)
            for _,name in ipairs(t) do
                per_person[name] = (per_person[name] or 0) + counts[name]
                if expand == curr_fail then
                    tooltip:AddLine(" "..Blue(name), tostring(counts[name]))
                    empty = false
                end
            end
        else
            for name,count in pairs(counts) do
                per_person[name] = (per_person[name] or 0) + count
            end
        end

        if expand == curr_fail then
            if empty then tooltip:AddLine(" "..(L["empty_failtype"])) end
            tooltip:AddHeader(" ")
        end
    end
    
    if curr_fail == 0 then tooltip:AddHeader(" ") end
    tooltip:AddHeader(HYellow(L["label_per_person"]), HYellow(tostring(grandtotal)))
    if curr_fail == 0 then
        local t = sortbyFails(per_person)
        for _, name in ipairs(t) do
        	tooltip:AddLine(Blue(" "..name), tostring(per_person[name]))
        end
    end

    tooltip:AddHeader(" ")
	tooltip:AddLine(L["tt_clickhint1L"].." "..L["tt_clickhint1R"])
	tooltip:AddLine(L["tt_clickhint2L"].." "..L["tt_clickhint2R"])
	tooltip:AddLine(L["tt_clickhint3L"].." "..L["tt_clickhint3R"])


	tooltip:EnableMouse()
	tooltip:SmartAnchorTo( self )
    tooltip:Show()
end

function DO.OnLeave(self)
    tooltip = LTT:Release(tooltip)
end

--[===[@debug@ 
FB = addon
--@end-debug@]===]

